#import "global.h"
#import "StageView.h"
#import "Enemy.h"

@implementation StageView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
        /* 배경 이미지 로드 */
        NSString* imageName = [[NSBundle mainBundle] pathForResource:@"background" ofType:@"png"];
        backgroundImage = [[NSImage alloc] initWithContentsOfFile:imageName];
		
        /* 우주선 이미지 로드 & 초기 좌표 설정 */
        imageName = [[NSBundle mainBundle] pathForResource:@"hero" ofType:@"png"];
        heroImage = [[NSImage alloc] initWithContentsOfFile:imageName];
        heroRect.size = [heroImage size];
        heroRect.origin = NSMakePoint((SCREEN_WIDTH - heroRect.size.width)/2, 0);
        
        /* 미사일 이미지 로드 & 초기 좌표 설정 */
        imageName = [[NSBundle mainBundle] pathForResource:@"missile" ofType:@"png"];
        missileImage = [[NSImage alloc] initWithContentsOfFile:imageName];
        missileRect.size = [missileImage size];
        missileRect.origin = NSMakePoint(heroRect.origin.x + heroRect.size.width/2 - missileRect.size.width/2, heroRect.size.height);
        
		/* 적기1 이미지 로드 */
        imageName = [[NSBundle mainBundle] pathForResource:@"enemy1" ofType:@"png"];
        enemyImage1 = [[NSImage alloc] initWithContentsOfFile:imageName];
		
		/* 적기2 이미지 로드 */
        imageName = [[NSBundle mainBundle] pathForResource:@"enemy2" ofType:@"png"];
        enemyImage2 = [[NSImage alloc] initWithContentsOfFile:imageName];
		
		/* 적기3 이미지 로드 */
        imageName = [[NSBundle mainBundle] pathForResource:@"enemy3" ofType:@"png"];
        enemyImage3 = [[NSImage alloc] initWithContentsOfFile:imageName];
				
		int i;
		
		/** 적기 오브젝트 생성 */
		for(i = 0; i < MAX_ENEMY1; i++)
		{
			Enemy1[i] = [[Enemy alloc] initWithImage:enemyImage1
												type:1];
		}
		for(i = 0; i < MAX_ENEMY2; i++)
		{
			Enemy2[i] = [[Enemy alloc] initWithImage:enemyImage2
												type:2];
		}
		for(i = 0; i < MAX_ENEMY3; i++)
		{
			Enemy3[i] = [[Enemy alloc] initWithImage:enemyImage3
												type:3];
		}
		
		/** 1 스테이지 설정 */
		curStage = 0;
		[self setStage];
        
		/* 30프레임으로 타이머 설정 */ 
        timer = [[NSTimer scheduledTimerWithTimeInterval: (1.0f / 30.0f) 
                                                  target: self 
                                                selector:@selector(processGame) 
                                                userInfo:self 
                                                 repeats:true] retain];		
	}
	return self;
}

/** 할당된 메모리 반환 */
- (void) dealloc 
{
	int i;
	
    [backgroundImage release];
    [heroImage release];
    [missileImage release];
	
	[enemyImage1 release];
	[enemyImage2 release];
	[enemyImage3 release];

	for(i = 0; i < MAX_ENEMY1; i++)
	{
		[Enemy1[i] release];
	}
	
	for(i = 0; i < MAX_ENEMY2; i++)
	{
		[Enemy2[i] release];
	}
	
	for(i = 0; i < MAX_ENEMY3; i++)
	{
		[Enemy3[i] release];
	}
	
    [super dealloc];
}

- (void) setStage
{
	int i;
	NSPoint pos;
	
	/* 현재 스테이지를 1 증가 시킨다. */
	curStage++;
		
	/* 적기의 좌/우 움직이는 속도와 방향 설정. */
	enemyDir = curStage;

	isFire = NO;
	
	/** 적기들의 초기 위치와 상태를 설정한다. */
	pos.x = 45;
	pos.y = 190;
	for(i = 0; i < MAX_ENEMY1; i++)
	{
		[Enemy1[i] setPosition:pos];
		[Enemy1[i] setState:1];
	
		pos.x += 45;
	}

	pos.x = 78;
	pos.y = 225;
	for(i = 0; i < MAX_ENEMY2; i++)
	{
		[Enemy2[i] setPosition:pos];
		[Enemy2[i] setState:1];
	
		pos.x += 55;
	}

	pos.x = 122;
	pos.y = 265;
	for(i = 0; i < MAX_ENEMY3; i++)
	{
		[Enemy3[i] setPosition:pos];
		[Enemy3[i] setState:1];
	
		pos.x += 80;
	}
}

/** 키입력을 받기 위한 설정 */
- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (void)keyDown:(NSEvent *)event
{
    int keyCode;
	
    /* 현재 눌려진 키값을 얻어 온다. */
    keyCode = [event keyCode];
    	
    /* 좌측으로 이동 */
    if(keyCode == 123)
    {
        heroRect.origin.x -= HERO_SPEED;
        if(heroRect.origin.x < 0)
            heroRect.origin.x = 0;
    }
	
    /* 우측으로 이동 */
    if(keyCode == 124)
    {    
        heroRect.origin.x += HERO_SPEED;
		
        if(heroRect.origin.x >= SCREEN_WIDTH - heroRect.size.width)
            heroRect.origin.x = SCREEN_WIDTH - heroRect.size.width;
    }
    
    /* 발사(스페이스) */
    if(keyCode == 49)
    {
        [self fireMissile];
    }
}

- (void)fireMissile
{
    if(isFire == NO)
    {
        /* 미사일이 처음 발사되었을 경우에 초기위치를 우주선 좌표로 설정 */
        missileRect.origin = NSMakePoint(heroRect.origin.x + heroRect.size.width/2 - missileRect.size.width/2, heroRect.size.height);
    }
    isFire = YES;
}

- (void)processGame
{
    if(isFire == YES)
    {
        /* 미사일이 발사중이면 y좌표를 이동 */
        missileRect.origin.y += MISSILE_SPEED;
        if(missileRect.origin.y >= SCREEN_HEIGHT)
        {
            /* 화면상단에 위치했을 경우에는 미사일을 출력하지 않는다 */
            isFire = NO;
        }
    }
    
    [self setNeedsDisplay:YES];
}

- (void)drawRect:(NSRect)rect
{
	NSRect imgRect;
    NSRect drawRect;
    
    /* 배경 이미지 출력 */
    imgRect.origin = NSZeroPoint;
    imgRect.size = [backgroundImage size];
    
    drawRect = [self bounds];
    
    [backgroundImage drawInRect:drawRect
					   fromRect:imgRect
					  operation:NSCompositeSourceOver
					   fraction:1.0];
    
    /* 우주선 출력 */
    imgRect.origin = NSZeroPoint;
    imgRect.size = [heroImage size];
    
    [heroImage drawInRect:heroRect
				 fromRect:imgRect
				operation:NSCompositeSourceOver
				 fraction:1.0];    
    
	
	int i;
	int isChange = 0; 
	int enemyCount = 0;
	
	/* 적기들을 출력하고 좌우 경계선을 넘어간 적기가 있을 경우를 체크한다. */
	for(i = 0; i < MAX_ENEMY1; i++)
	{
		if([Enemy1[i] state])
		{
			enemyCount++;
			isChange |= [Enemy1[i] moveAndDisplay:enemyDir];
		}	
	}

	for(i = 0; i < MAX_ENEMY2; i++)
	{
		if([Enemy2[i] state])
		{
			enemyCount++;
			isChange |= [Enemy2[i] moveAndDisplay:enemyDir];
		}	
	}
	
	for(i = 0; i < MAX_ENEMY3; i++)
	{
		if([Enemy3[i] state])
		{
			enemyCount++;
			isChange |= [Enemy3[i] moveAndDisplay:enemyDir];
		}
	}
	
	/** 남은 적기가 없을 경우에는 다음 스테이지로 넘어 간다. */
	if(enemyCount == 0)
	{
		[self setStage];
		return;	
	}
		
    if(isFire == YES)
    {
        /* 미사일 출력 */
        imgRect.origin = NSZeroPoint;
        imgRect.size = [missileImage size];
        
        [missileImage drawInRect:missileRect
						fromRect:imgRect
					   operation:NSCompositeSourceOver
						fraction:1.0];    
    }
    
	/** 적기가 좌우측의 경계를 넘었을 경우에는 아래로 이동한다. */
	if(isChange)
	{
		enemyDir *= -1;
		for(i = 0; i < MAX_ENEMY1; i++)
		{
			if([Enemy1[i] state])
				[Enemy1[i] down];
		}
		
		for(i = 0; i < MAX_ENEMY2; i++)
		{
			if([Enemy2[i] state])
				[Enemy2[i] down];
		}
		
		for(i = 0; i < MAX_ENEMY3; i++)
		{
			if([Enemy3[i] state])
				[Enemy3[i] down];
		}
	}
}
@end
